@@ -5,15 +5,9 @@ import time
 from threading import Event, Thread
 from typing import Literal, Tuple
 
-try:
-    import pyinterface
-except ImportError:
-    pyinterface = None
-
-from neclib import config
 from .pulse_controller_base import PulseController
 from ...exceptions import ConfigurationError
-from ... import utils
+from ... import config, get_logger, utils
 
 
 class CPZ7415V(PulseController):
@@ -25,18 +19,22 @@ class CPZ7415V(PulseController):
         non-zero, when multiple PCI board of same model are mounted on a single FA
         (Factory Automation) controller. Accepted values are ``[0, 1, ..., 16]`` if
         given as ``int``, ``["0", "1", ..., "9", "A", ..., "F"]`` if ``str``.
-    - conf
-        Board configuration.
+    - other parameters
+        See <http://www.interface.co.jp/download/tutorial/tut0053_14.pdf>.
 
     """
 
     Manufacturer = "Interface"
     Model = "CPZ7415V"
 
+    CommandQueueMaxSize = {"warning": 10, "fatal": 30}
+
     def __init__(self) -> None:
         if config.antenna_cpz7415v is None:
             raise ConfigurationError("Parameters for CPZ-7415V not configured at all.")
 
+        self.logger = get_logger(__name__)
+
         self.rsw_id = int(config.antenna_cpz7415v_rsw_id)
         self.do_status = config.antenna_cpz7415v_do_conf
         self.use_axes = config.antenna_cpz7415v_useaxes.lower()
@@ -68,6 +66,8 @@ class CPZ7415V(PulseController):
 
     @utils.skip_on_simulator
     def _initialize_io(self):
+        import pyinterface
+
         io = pyinterface.open(7415, self.rsw_id)
         for ax in self.use_axes:
             io.set_pulse_out(ax, "method", [self.pulse_conf[ax]])
@@ -99,10 +99,29 @@ class CPZ7415V(PulseController):
             self.current_moving = {ax: v for ax, v in zip(self.use_axes, moving)}
 
             if not self.task_queue.empty():
-                task = self.task_queue.get()
-                task["func"](task["args"], task["axis"])
+                if self._check_realtimeness(self.task_queue):
+                    task = self.task_queue.get()
+                    task["func"](task["args"], task["axis"])
+                else:
+                    # Commands in task_queue are too outdated, so it's dangerous to
+                    # execute them.
+                    _ = [self._stop(None, ax) for ax in self.use_axes]
             time.sleep(1e-5)
 
+    def _check_realtimeness(self, queue_: queue.Queue) -> bool:
+        qsize = queue_.qsize()
+        if qsize < self.CommandQueueMaxSize["warning"]:
+            return True
+        elif qsize < self.CommandQueueMaxSize["fatal"]:
+            self.logger.warning(
+                f"{qsize} commands remain unprocessed. "
+                "Consider reducing command frequency `antenna_command_frequency`."
+            )
+            return True
+        else:
+            self.logger.fatal(f"{qsize} commands remain unprocessed. Emergency stop...")
+            return False
+
     def _convert_ax(self, axis: Literal["az", "el"]) -> str:
         if axis in self.use_axes:
             return axis
@@ -120,10 +139,11 @@ class CPZ7415V(PulseController):
                 self.task_queue.put(
                     {"func": self._start, "args": speed_step, "axis": ax}
                 )
-        return
 
-    def set_speed(self, speed: float, axis: str) -> None:
+    def set_speed(self, speed: float, axis: Literal["az", "el"]) -> None:
         ax = self._convert_ax(axis)
+        speed *= config.antenna_speed_to_pulse_factor[axis]
+
         if abs(speed) < self.low_speed[ax]:
             self.task_queue.put({"func": self._stop, "args": None, "axis": ax})
             self.last_direction[ax] = 0
@@ -183,4 +203,7 @@ class CPZ7415V(PulseController):
 
     def get_step(self, axis: Literal["az", "el"]) -> int:
         ax = self._convert_ax(axis)
-        return self.current_step[ax]
+        return int(self.current_step[ax])
+
+    def finalize(self) -> None:
+        self._output_do([0, 0, 0, 0])
